#include <imsls.h>
#include <stdio.h>

int main(){
   int i, j;                           /* index variables            */
   int n = 40;                         /* population size            */
   int n_generations = 0;              /* final number of generations*/
   int n_real = 2;                     /* number of real phenotypes  */
   int   r_intervals[2] = {65536, 65536};
   float r_bounds[4]    = {-2.048, 2.048, -2.048, 2.048};
   float* genStats;                    /* generation statistics      */
   static float deJongR2(Imsls_f_individual* individual);
   Imsls_f_chromosome* chromosome;     /* chromosome data structure  */
   Imsls_f_individual* best_individual;/* optimum                    */
   Imsls_f_population* population;     /* population data structure  */
   Imsls_f_population* last_generation;/* last generation            */
   char *dashes=
      "**************************************************************";
   /******************************************************************/
   /* In this example the user function is thread safe.  Let CNL     */
   /* know it is safe, which allows genetic algorithm to run in      */
   /* parallel, if that capability exists on the user computer.      */
   /******************************************************************/
   imsls_omp_options(IMSLS_SET_FUNCTIONS_THREAD_SAFE, 1, 0);
   imsls_random_seed_set(12345);
   chromosome = imsls_f_ga_chromosome( 
      IMSLS_REAL, n_real, r_intervals, r_bounds, 0);
   population = imsls_f_ga_random_population(n, chromosome, 
      IMSLS_GRAY_ENCODING,
      IMSLS_FITNESS_FCN, deJongR2, 0);

   best_individual = imsls_f_genetic_algorithm(deJongR2, population, 
      IMSLS_PRINT_LEVEL, IMSLS_FINAL,
      IMSLS_MAX_FITNESS, 3999.999,
      IMSLS_CROSSOVER_PROB, 0.6,
      IMSLS_GRAY_ENCODING, 
      IMSLS_MAX_GENERATIONS, 1000,
      IMSLS_N_GENERATIONS, &n_generations,
      IMSLS_GENERATION_STATS, &genStats,
      IMSLS_LAST_GENERATION,&last_generation, 0);
   printf(
      "\n*****************GENERATION STATISTICS*****************\n");
   printf("Generation  Max. Fit.   Avg. Fit.  Min. Fit.     CV\n");
   printf("*******************************************************\n");
   for(i=0; i<=n_generations; i++){
      printf("Gen. %3d: %11.5f %10.2f %10.2f %9.2f \n",
         i, genStats[4*i], genStats[4*i+2], genStats[4*i+1],
         100*genStats[4*i+3]/genStats[4*i+2]);
   }
   printf("\nLAST GENERATION\n");
   printf("%s\n",dashes);
   printf("\nIndv  Fitness              ");
   printf("Chromosome              X1     X2\n");
   for(i=0; i<last_generation->n; i++){
      printf(" %2d   %6.2f  ", i, last_generation->fitness[i]);
      for(j=0; j<last_generation->chromosome->c_length; j++) 
         printf("%d", 
         last_generation->individual[i]->chromosome->allele[j]);
      printf("%7.3f %6.3f\n", 
         last_generation->individual[i]->realPhenotype[0],
         last_generation->individual[i]->realPhenotype[1]);
   }
   printf("\nMaximum:    %6.2f for Individual %d\n", 
      last_generation->maxFitness,
      last_generation->indexFittest);
   printf("Minimum:    %6.2f for Individual %d\n", 
      last_generation->minFitness,
      last_generation->indexWeakest);
   printf("Average:    %6.2f\n", last_generation->avgFitness);
   printf("Std. Dev:    %6.2f\n\n", last_generation->stdFitness);
   printf("%s\n", dashes);
}
/*********************************************************************/
/* De Jong's R2 Function                                             */
/*********************************************************************/
static float deJongR2(Imsls_f_individual* individual)
{
   float f, x1, x2;
   x1 = individual->realPhenotype[0];
   x2 = individual->realPhenotype[1];
   f = 100*(x1*x1-x2)*(x1*x1-x2) + (1.0-x1)*(1.0-x1);
   f = 4000 - f;
   return f;
}
